﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web.Script.Services;
using System.Web.Services;
using System.Xml;

/// <summary>
/// Descripción breve de bicingApp
/// </summary>
[WebService(Namespace = "http://localhost/", Name="ServeiBicing")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
[System.Web.Script.Services.ScriptService]
public class bicisXTots : System.Web.Services.WebService {

    //const double RadiTerra = 6378.137;
    const float RadiTerra = 6378F;
    DateTime DataActualitzacio { get; set; }
    List<Estacio> LlistaEstacions { get; set; }

    public bicisXTots()
    {

    }

    [WebMethod]
    public Estacions ObtenirDadesEstacions(string lat, string lng)
    {
        Estacions _estacions = new Estacions();

        XmlDocument _dades = new XmlDocument();
        _dades.Load("http://wservice.viabicing.cat/getstations.php?v=1");

        _estacions.DataActualitzacio = _dades.DocumentElement.SelectSingleNode("updatetime").InnerText;

        var _estacions2 = ObtenirEstacions(_dades, lat, lng).OrderBy(e => e.Distancia).ToList();

        _estacions.LlistaEstacions = _estacions2;

        return _estacions;
    }

    List<Estacio> ObtenirEstacions(XmlDocument document, string lat, string lng)
    {
        var _result = new List<Estacio>();

        var _llistaElements = document.DocumentElement.SelectNodes("station");
        foreach (XmlElement _element in _llistaElements)
        {
            var _estacio = new Estacio();
            _estacio.Identificador = _element.SelectSingleNode("id").InnerText.PadLeft(3, '0');
            _estacio.Latitud = Convert.ToSingle(_element.SelectSingleNode("lat")
                .InnerText.Replace(",", "."), CultureInfo.GetCultureInfo("en"));
            _estacio.Longitud = Convert.ToSingle(_element.SelectSingleNode("long")
                .InnerText.Replace(",", "."), CultureInfo.GetCultureInfo("en"));
            _estacio.Carrer = _element.SelectSingleNode("street").InnerText;
            _estacio.Alcada = _element.SelectSingleNode("height")
                .InnerText;
            _estacio.NumeroCarrer = _element.SelectSingleNode("streetNumber")
                .InnerText;
            _estacio.EstacionsProperes = new List<int>(_element.SelectSingleNode(
                "nearbyStationList").InnerText.Split(',').Select(int.Parse));
            _estacio.Estat = _element.SelectSingleNode("status").InnerText;
            _estacio.BicisLliures = Convert.ToInt32(_element.SelectSingleNode("bikes")
                .InnerText);
            _estacio.AparcamentsLliures = Convert.ToInt32(_element.SelectSingleNode(
                "slots").InnerText);
            _estacio.Distancia = ObtenirDistancia(Convert.ToSingle(lat.Replace(",",
                "."), CultureInfo.GetCultureInfo("en")), _estacio.Latitud,
                Convert.ToSingle(lng.Replace(",", "."), CultureInfo.GetCultureInfo("en")),
                _estacio.Longitud);
            _result.Add(_estacio);
        }

        return _result;
    }

    static float ObtenirDistancia(float latDisp, float latEst, float lngDisp,
        float lngEst)
    {
        var difLat = EnRadianes(latDisp - latEst);
        var difLng = EnRadianes(lngDisp - lngEst);
        var a = Math.Sin(Math.Pow(difLat / 2, 2)) +
                Math.Cos(EnRadianes(latDisp)) *
                Math.Cos(EnRadianes(latEst)) *
                Math.Sin(Math.Pow(difLng / 2, 2));
        var c = 2 * Math.Atan2(Math.Sqrt(a), Math.Sqrt(1 - a));

        return RadiTerra * Convert.ToSingle(c) * 1000;
    }

    static float EnRadianes(float valor)
    {
        return Convert.ToSingle(Math.PI / 180) * valor;
    }

}
